/*
 * Entry.java
 */
package de.holzem.refbuilder;

import java.util.Iterator;
import java.util.Locale;
import java.util.TreeMap;

/**
 * Entry
 */
public class Entry extends TreeMap
{
	private static final Locale locale = new Locale("de");
	String name;
	String shortName;

	public Entry(String name, String shortName)
	{
		this.name = name;
		this.shortName = shortName;
	}

	public void add(String source, String title)
	{
		Reference ref = (Reference)get(source);
		if (ref == null) {
			ref = new Reference(source, title);
			String key = title.toUpperCase(locale).replace("", "AE").replace("",
					"OE").replace("", "UE");
			put(key, ref);
		}
	}

	/**
	 * Convert content to XML
	 * 
	 * @return XML-representation
	 */
	public String toXml()
	{
		StringBuffer sb = new StringBuffer();
		sb.append("<entry name=\"").append(name).append("\" shortname=\"").append(
				shortName).append("\">\n");
		for (Iterator i = values().iterator(); i.hasNext();) {
			Reference ref = (Reference)i.next();
			sb.append(ref.toXml()).append("\n");
		}
		sb.append("</entry>\n");
		return sb.toString();
	}

	/**
	 * @see java.lang.Comparable#compareTo(java.lang.Object)
	 */
	public int compareTo(Object arg0)
	{
		Entry other = (Entry)arg0;
		return name.toUpperCase().compareTo(other.name.toUpperCase());
	}

	/**
	 * @see java.lang.Object#toString()
	 */
	public String toString()
	{
		StringBuffer sb = new StringBuffer();
		sb.append("Entry (").append(name).append(")");
		return sb.toString();
	}
}