/*
 * RefBuilderTask.java
 */
package de.holzem.refbuilder;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.DirectoryScanner;
import org.apache.tools.ant.Project;
import org.apache.tools.ant.taskdefs.MatchingTask;

/**
 * RefBuilderTask
 */
public class RefBuilderTask extends MatchingTask
{
	private File basedir;
	private File destdir;

	/**
	 * @see org.apache.tools.ant.Task#init()
	 */
	public void init() throws BuildException
	{
		super.init();
	}

	/**
	 * @see org.apache.tools.ant.Task#execute()
	 */
	public void execute() throws BuildException
	{
		verifyValues();
		log("Running RefBuilderTask on " + basedir, Project.MSG_INFO);
		log("  Generated Output goes into " + destdir, Project.MSG_INFO);
		DirectoryScanner ds = getDirectoryScanner(basedir);
		String[] files = ds.getIncludedFiles();
		createThumbRefList(files);
		createLocationIndex(files);
		createTopicIndex(files);
	}

	private void createThumbRefList(String[] files)
	{
		ThumbRefParser thumbRefParser = new ThumbRefParser(this, basedir, files);
		thumbRefParser.parse();
		ThumbRefList list = thumbRefParser.getThumbRefList();
		//log("Thumbrefs:\n" + list.toXml(), Project.MSG_INFO);
		String thumbreffile = destdir + "\\" + "thumbref.xml";
		String output = "<?xml version=\"1.0\" encoding=\"iso-8859-1\"?>\n" + list.toXml();
		FileWriter fw = null;
		try {
			fw = new FileWriter(thumbreffile);
			fw.write(output);
			fw.flush();
			fw.close();
		} catch (IOException e) {
			log("Unable to write "+thumbreffile, Project.MSG_ERR);
		}
	}

	private void createLocationIndex(String[] files)
	{
		LocationParser lp = new LocationParser(this, basedir, files);
		lp.parse();
		Index idx = lp.getIndex();
		//log("Ortsindex:\n" + idx.toXml(), Project.MSG_INFO);
		String idxfile = destdir + "\\" + "ortsindex.xml";
		String output = "<?xml version=\"1.0\" encoding=\"iso-8859-1\"?>\n" + idx.toXml();
		FileWriter fw = null;
		try {
			fw = new FileWriter(idxfile);
			fw.write(output);
			fw.flush();
			fw.close();
		} catch (IOException e) {
			log("Unable to write "+idxfile, Project.MSG_ERR);
		}
	}

	private void createTopicIndex(String[] files)
	{
		IndexParser ip = new IndexParser(this, basedir, files);
		ip.parse();
		Index idx = ip.getIndex();
		//log("Themenindex:\n" + idx.toXml(), Project.MSG_INFO);
		String idxfile = destdir + "\\" + "themenindex.xml";
		String output = "<?xml version=\"1.0\" encoding=\"iso-8859-1\"?>\n" + idx.toXml();
		FileWriter fw = null;
		try {
			fw = new FileWriter(idxfile);
			fw.write(output);
			fw.flush();
			fw.close();
		} catch (IOException e) {
			log("Unable to write "+idxfile, Project.MSG_ERR);
		}
	}

	/**
	 * Check mandatory values for RebBuilderTask
	 * 
	 * @throws BuildException
	 */
	private void verifyValues() throws BuildException
	{
		if (basedir == null) {
			throw new BuildException(this.getClass().getName()
					+ ": basedir must be specified");
		}
		if (destdir == null) {
			throw new BuildException(this.getClass().getName()
					+ ": destdir must be specified");
		}
	}

	/**
	 * @param basedir
	 *          The basedir to set.
	 */
	public void setBasedir(File basedir)
	{
		this.basedir = basedir;
	}

	/**
	 * @param destdir
	 *          The destdir to set.
	 */
	public void setDestdir(File destdir)
	{
		this.destdir = destdir;
	}
}