/*
 * ThumbRefParser.java
 */
package de.holzem.refbuilder;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.InputStream;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Stack;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;
import org.apache.tools.ant.Project;
import org.apache.tools.ant.Task;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;

/**
 * ThumbRefParser
 */
public class ThumbRefParser
{
	Task task;
	ThumbRefList list;
	String[] files;
	File dir;

	/**
	 * Constructor
	 * 
	 * @param task
	 * @param dir
	 * @param files
	 */
	public ThumbRefParser(Task task, File dir, String[] files)
	{
		this.task = task;
		this.dir = dir;
		this.files = files;
		this.list = new ThumbRefList();
	}

	/**
	 * parse list of files for thumbrefs
	 */
	public void parse()
	{
		task.log("  Parsing for thumbrefs", Project.MSG_INFO);
		for (int i = 0; i < files.length; i++) {
			String file = files[i];
			String path = dir + "\\" + file;
			//task.log("  Parsing for thumbref: " + file);
			parseFile(path, file);
		}
		list.sort();
	}

	/**
	 * parse one file for thumbrefs
	 * 
	 * @param path
	 *          path to file
	 * @param file
	 *          filename for website-relative reference
	 */
	private void parseFile(String path, String file)
	{
		InputStream is = null;
		try {
			is = new FileInputStream(path);
		} catch (FileNotFoundException e) {
			task.log(e.getClass().getName() + ":\n" + e.getMessage());
			return;
		}
		try {
			SAXParserFactory spf = SAXParserFactory.newInstance();
			spf.setValidating(false);
			spf.setFeature("http://xml.org/sax/features/namespaces", false);
			spf.setFeature("http://xml.org/sax/features/validation", false);
			spf.setFeature(
					"http://apache.org/xml/features/nonvalidating/load-external-dtd",
					false);
			SAXParser saxParser = spf.newSAXParser();
			ThumbRefHandler handler = new ThumbRefHandler(task, file);
			saxParser.parse(is, handler);
			list.addAll(handler.list);
		} catch (Exception e) {
			task
					.log(e.getClass().getName() + ":\n" + e.getMessage(), Project.MSG_ERR);
		}
	}

	/**
	 * @return the parsed ThumbRefList
	 */
	public ThumbRefList getThumbRefList()
	{
		return list;
	}

	/**
	 * ThumbRefHandler: a SAXHandler for thumbrefs
	 */
	private static class ThumbRefHandler extends DefaultHandler
	{
		Task task = null;
		String file = null;
		Stack stack = null;
		StringBuffer value = null;
		String title = null;
		ThumbRefList list = null;
		Map foundMap = null;

		ThumbRefHandler(Task task, String file)
		{
			this.task = task;
			this.file = "/" + file.replace('\\', '/').replace(".xml", ".html");
			this.stack = new Stack();
			this.value = new StringBuffer();
			this.list = new ThumbRefList();
			this.foundMap = new HashMap(101);
		}

		/**
		 * @see org.xml.sax.ContentHandler#characters(char[], int, int)
		 */
		public void characters(char[] ch, int start, int length)
				throws SAXException
		{
			String trimValue = new StringBuffer().append(ch, start, length)
					.toString().trim();
			value.append(" ").append(trimValue);
		}

		/**
		 * @see org.xml.sax.ContentHandler#endElement(java.lang.String,
		 *      java.lang.String, java.lang.String)
		 */
		public void endElement(String uri, String localName, String qName)
				throws SAXException
		{
			if (stack.toString().equals("[document, header, title]")) {
				title = value.toString().trim().replace("\"", "");
			}
			String element = (String)stack.pop();
		}

		/**
		 * @see org.xml.sax.ContentHandler#startElement(java.lang.String,
		 *      java.lang.String, java.lang.String, org.xml.sax.Attributes)
		 */
		public void startElement(String uri, String localName, String qName,
				Attributes attributes) throws SAXException
		{
			if (qName.equals("titlethumb")) {
				String href = attributes.getValue("href");
				String map = href.substring(href.lastIndexOf('/') + 1);
				
				if (foundMap.get(map) == null) {
					ThumbRef thumb = new ThumbRef(file, map, title);
					list.add(thumb);
					foundMap.put(map, thumb);
				}
			}
			stack.push(qName);
			value.setLength(0);
		}

		/**
		 * @see org.xml.sax.ContentHandler#endDocument()
		 */
		public void endDocument() throws SAXException
		{
			for (Iterator i = list.iterator(); i.hasNext();) {
				ThumbRef ref = (ThumbRef)i.next();
				ref.setTitle(title);
			}
		}
	}
}